﻿using System;
using System.Windows.Forms;

namespace SD_MMC_CRC7
{
    public partial class frmMain : Form
    {
        public const byte CRC7_POLY = 0x89;
        public static byte[] crctable = new byte[256];

        public frmMain()
        {
            InitializeComponent();
            this.ActiveControl = tbData;
        }

        private void tbData_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (!Char.IsDigit(e.KeyChar) &&
                !Char.IsControl(e.KeyChar) &&
                !((e.KeyChar >= 'A') && (e.KeyChar <= 'F')) &&
                !((e.KeyChar >= 'a') && (e.KeyChar <= 'f')) &&
                !(e.KeyChar == '-') && !(e.KeyChar == ' '))
                e.Handled = true; //do not process keypress event
        }

        private void btnCalc_Click(object sender, EventArgs e)
        {
            byte[] CID = new byte[15];
            byte i, tcrc;

            if (tbData.Text.Replace("-", "").Replace(" ", "").Length >= 30)
            {
                //read input data
                Array.Copy(HexStringToByteArray(tbData.Text), CID, 15);
                //Generate CRC table
                GenerateCRCTable();
                // Calculate CRC7
                tcrc = 0;
                for (i = 0; i < 15; i++) // for all 15 data bytes...
                {
                    tcrc = AddByteToCRC(tcrc, CID[i]); // add byte to CRC
                }
                //Add 1 according specification
                tcrc = (byte)((tcrc << 1) + 1); // format the CRC7 value and show it
                //Show result
                tbResult.Text = tcrc.ToString("X2");
            }
            else
            {
                MessageBox.Show("The CID is not complete!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        static void GenerateCRCTable()
        {
            int i, j;

            //Generate a table value for all 256 possible byte values
            for (i = 0; i < 256; i++)
            {
                crctable[i] = ((i & 0x80) != 0) ? (byte)(i ^ CRC7_POLY) : (byte)i;
                for (j = 1; j < 8; j++)
                {
                    crctable[i] <<= 1;
                    if ((crctable[i] & 0x80) != 0)
                        crctable[i] ^= CRC7_POLY;
                }
            }
        }

        static byte AddByteToCRC(byte crc, byte b)
        {
            return crctable[(crc << 1) ^ b];
        }

        public static byte[] HexStringToByteArray(String hex)
        {
            hex = (hex.Replace("-", "")).Replace(" ", "");
            int NumberChars = hex.Length;
            byte[] bytes = new byte[NumberChars / 2];
            for (int i = 0; i < NumberChars; i += 2)
                bytes[i / 2] = Convert.ToByte(hex.Substring(i, 2), 16);
            return bytes;
        }


    }
}
